
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.012.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "OAI-PMH provider backed by a searchable Catmandu::Store",
  "dist_author" => [
    "Nicolas Steenlant, C<< <nicolas.steenlant at ugent.be> >>"
  ],
  "dist_name" => "Dancer-Plugin-Catmandu-OAI",
  "dist_version" => "0.0506",
  "license" => "perl",
  "module_name" => "Dancer::Plugin::Catmandu::OAI",
  "recursive_test_files" => 1,
  "requires" => {
    "Catmandu" => "1.0",
    "Catmandu::Exporter::Template" => "0.11",
    "Clone" => "0.36",
    "Dancer" => "1.3123",
    "Data::MessagePack" => "1.00",
    "DateTime" => "1.08",
    "DateTime::Format::ISO8601" => 0,
    "DateTime::Format::Strptime" => 0,
    "MIME::Base64" => "3.15",
    "perl" => "v5.10.1"
  },
  "test_requires" => {
    "Dancer::Test" => "1.3123",
    "Test::More" => "0.88"
  }
);


my %fallback_build_requires = (
  "Dancer::Test" => "1.3123",
  "Module::Build" => "0.28",
  "Test::More" => "0.88"
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
